# package for explicit semantic analysis
from iobio.qnorm import QNorm
from pandas import concat as pd_concat

class ReferenceCorrelation:
    def __init__(self,data,reference,quantile_normalize=True,min_variance=0,corr_method='pearson'):
        self.quantile_normalize = quantile_normalize

        data_index = data.index
        reference_index = reference.index
        shared = data.index.intersection(reference.index)
        self._data = data.loc[shared]
        self._reference = reference.loc[shared]

        data_index = self._data.var(1)[min_variance<self._data.var(1)].index
        reference_index = self._reference.var(1)[min_variance<self._reference.var(1)].index
        self._shared = data_index.intersection(reference_index)
        self._data = self._data.loc[self._shared].copy()
        self._reference = self._reference.loc[self._shared].copy()
        if quantile_normalize:
            qfit = QNorm().fit(self._reference)
            self._data = qfit.transform(self._data)
            self._reference = qfit.transform(self._reference)
        self._corr = pd_concat([self._data,self._reference],1).corr(method=corr_method).iloc[self._data.shape[1]:].reset_index(drop=True)
    @property
    def features_selected(self):
        return self._shared
    @property
    def reference_correlations(self):
        return self._corr.iloc[:,self._data.shape[1]:].copy()
    @property
    def data_correlations(self):
        return self._corr.iloc[:,0:self._data.shape[1]].copy()
    def correlate(data,reference):
        self.reference = reference
        self.qfit = QNorm().fit(self.reference)
        return



# Adapted from: https://stackoverflow.com/questions/37935920/quantile-normalization-on-pandas-dataframe
# Citing: https://en.wikipedia.org/wiki/Quantile_normalization
class QNorm:
    def __init__(self):
        return
    def fit(self,df):
        return _QNormFit(self,df.stack().groupby(df.rank(method='first').stack().astype(int)).mean())
    def fit_transform(self,df):
        return fit(df).transform(df)
class _QNormFit:
    def __init__(self,parent,df):
        self.qnorm = parent
        self.rank_mean = df
        return
    def transform(self,df):
        return df.rank(method='min').stack().astype(int).map(self.rank_mean).unstack()




from sklearn.cluster import AgglomerativeClustering
import numpy as np
import pandas as pd
import seaborn as sns
from scipy.stats import zscore
#from collections import namedtuple
#OrderedCluster = namedtuple('OrderedCluster',['ids','colors'])
def cluster_ordered_agglomerative (dataframe,
	                               n_clusters=2,
	                               affinity='euclidean',
	                               memory=None,
	                               connectivity=None,
	                               compute_full_tree='auto',
	                               linkage='ward',
	                               rankstat='median',
	                               cmapcolors='Blues',
                                       z_score=False):
    if z_score: dataframe = dataframe.copy().apply(zscore)
    clusters = AgglomerativeClustering(n_clusters=n_clusters,
    	                               affinity=affinity,
    	                               memory=memory,
    	                               connectivity=connectivity,
    	                               compute_full_tree=compute_full_tree,
    	                               linkage=linkage).\
        fit_predict(dataframe)
    clusters = pd.DataFrame({'unordered_id':clusters})
    clusters.index = dataframe.index
    rowsums = pd.DataFrame(dataframe.sum(1)).rename(columns={0:'rankstat'})
    clusters = clusters.merge(rowsums,left_index=True,right_index=True)
    rankfunc = None
    if rankstat == 'median': rankfunc = np.median
    elif rankstat == 'mean': rankfunc = np.mean
    else: raise ValueError("unsupported rankstat: "+str(rankstat))
    ordered = list(clusters.groupby('unordered_id').\
    	apply(lambda x: rankfunc(x['rankstat'])).sort_values().index)
    converter = dict(zip(ordered,range(0,n_clusters)))
    cluster_ids = clusters.apply(lambda x: converter[x['unordered_id']],1)
    clusters['cluster_id'] = cluster_ids
    colorconverter = sns.color_palette(cmapcolors,n_clusters)
    cluster_colors = clusters.apply(lambda x: colorconverter[int(x['cluster_id'])],1)
    clusters['cluster_colors'] = cluster_colors

    ordered_index = clusters.sort_values(['cluster_id','rankstat']).index
    dfo = pd.DataFrame({'ordered_index':ordered_index})
    dfo['order_index'] = range(0,dfo.shape[0])
    dfo = dfo.set_index('ordered_index')
    saveindex = clusters.index
    clusters = clusters.merge(dfo,left_index=True,right_index=True)
    clusters = clusters.loc[saveindex]
    return clusters.drop(columns=['unordered_id'])



from setuptools import setup, find_packages
from codecs import open
from os import path

this_folder = path.abspath(path.dirname(__file__))
with open(path.join(this_folder,'README.md'),encoding='utf-8') as inf:
  long_description = inf.read()

setup(
  name='iobio',
  version='0.1.0',
  description='Simple tools for analysis',
  long_description=long_description,
  url='https://github.com/jason-weirather/io-bio',
  author='Jason L Weirather',
  author_email='JasonL_Weirather@dfci.harvard.edu',
  license='Apache License, Version 2.0',
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
    'License :: OSI Approved :: Apache Software License'
  ],
  keywords='bioinformatics',
  packages=['iobio','iobio.qnorm','iobio.entropy','iobio.explicitsemanticanalysis']
)



import math
import numpy as np
def correlation_matrix_to_entropy(mat):
    mat = mat.copy().applymap(abs)
    mat.index = range(0,mat.shape[0])
    mat.columns = range(0,mat.shape[1])
    mat.columns.name = 'id_2'
    mat.index.name = 'id_1'
    tdf = mat.unstack().reset_index().rename(columns={0:'correlation'})
    tdf = tdf[tdf['id_2']<tdf['id_1']]
    tdf['corrlog'] = np.log(tdf['correlation'].add(0.0000001)) #can't have zero here
    return np.multiply(tdf['correlation'],tdf['corrlog']).multiply(-1).astype(float).sum()/tdf.shape[0]
    
